﻿Imports System.Drawing
Imports System.IO
Imports System.Windows.Forms
Imports System.Collections.ObjectModel
Imports ThinkGeo.MapSuite.Core

Public Class Form1

    Private mapEngine As New MapEngine()
    Private bitmap As Bitmap = Nothing
    Private count1 As Integer = 0
    Private count2 As Integer = 0
    Private pointShapes1 As Collection(Of PointShape)
    Private pointShapes2 As Collection(Of PointShape)


    Private Sub ToolBar_ButtonClick(ByVal sender As System.Object, ByVal e As System.Windows.Forms.ToolBarButtonClickEventArgs) Handles ToolBar.ButtonClick
        Select Case e.Button.Tag.ToString()
            Case "Zoom In"
                MapEngine.CurrentExtent.ScaleDown(50)
                Exit Select
            Case "Zoom Out"
                MapEngine.CurrentExtent.ScaleUp(50)
                Exit Select
            Case "Full Extent"
                MapEngine.CurrentExtent = ExtentHelper.GetDrawingExtent(New RectangleShape(-66.8581, 10.4995, -66.8456, 10.4915), Map.Width, Map.Height)
                Exit Select
            Case "Pan Left"
                MapEngine.CurrentExtent = ExtentHelper.Pan(MapEngine.CurrentExtent, PanDirection.Left, 20)
                Exit Select
            Case "Pan Right"
                MapEngine.CurrentExtent = ExtentHelper.Pan(MapEngine.CurrentExtent, PanDirection.Right, 20)
                Exit Select
            Case "Pan Up"
                MapEngine.CurrentExtent = ExtentHelper.Pan(MapEngine.CurrentExtent, PanDirection.Up, 20)
                Exit Select
            Case "Pan Down"
                MapEngine.CurrentExtent = ExtentHelper.Pan(MapEngine.CurrentExtent, PanDirection.Down, 20)
                Exit Select
            Case Else
                Exit Select
        End Select
        DrawImage()

    End Sub

    Private Sub DrawImage()
        If Bitmap IsNot Nothing Then
            Bitmap.Dispose()
        End If
        Bitmap = New Bitmap(Map.Width, Map.Height)
        MapEngine.OpenAllLayers()
        MapEngine.DrawDynamicLayers(Bitmap, GeographyUnit.DecimalDegree)
        MapEngine.CloseAllLayers()
        Map.Image = Bitmap
    End Sub


    Private Sub btnClose_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnClose.Click
        Me.Close()
    End Sub

    Private Function ReadTextFile(ByVal textFile As String) As Collection(Of PointShape)
        Dim pointShapes As New Collection(Of PointShape)()
        Dim SR As StreamReader = File.OpenText(textFile)
        Dim S As String
        S = SR.ReadLine()
        While S IsNot Nothing
            Dim X As Double = System.Convert.ToDouble(S.Substring(0, 8))
            Dim Y As Double = System.Convert.ToDouble(S.Substring(9, 7))
            Dim Angle As Double = System.Convert.ToDouble(S.Substring(17, 3))
            Dim pointShape As New PointShape(X, Y, Angle)
            pointShapes.Add(pointShape)

            S = SR.ReadLine()
        End While
        SR.Close()
        Return pointShapes
    End Function


    Private Sub timer1_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles timer1.Tick
        If count1 = pointShapes1.Count Then
            count1 = 0
        End If
        If count2 = pointShapes2.Count Then
            count2 = 0
        End If

        Try
            Dim inMemoryFeatureLayer As InMemoryFeatureLayer = DirectCast(MapEngine.DynamicLayers(0), InMemoryFeatureLayer)

            Dim feature As Feature = inMemoryFeatureLayer.InternalFeatures("Vehicle1")
            feature.ColumnValues("Angle") = System.Convert.ToString(pointShapes1(count1).Z)
            Dim pointShape1 As PointShape = TryCast(feature.GetShape(), PointShape)
            pointShape1.X = pointShapes1(count1).X
            pointShape1.Y = pointShapes1(count1).Y
            pointShape1.Id = "Vehicle1"

            Dim feature2 As Feature = inMemoryFeatureLayer.InternalFeatures("Vehicle2")
            feature2.ColumnValues("Angle") = System.Convert.ToString(pointShapes2(count2).Z)
            Dim pointShape2 As PointShape = TryCast(feature2.GetShape(), PointShape)
            pointShape2.X = pointShapes2(count2).X
            pointShape2.Y = pointShapes2(count2).Y
            pointShape2.Id = "Vehicle2"

            inMemoryFeatureLayer.Open()
            inMemoryFeatureLayer.EditTools.BeginTransaction()

            inMemoryFeatureLayer.EditTools.Update(pointShape1)
            inMemoryFeatureLayer.EditTools.Update(pointShape2)
            inMemoryFeatureLayer.EditTools.CommitTransaction()

            inMemoryFeatureLayer.Close()
        Finally
            count1 = count1 + 1
            count2 = count2 + 1
        End Try
        DrawImage()

    End Sub

    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        mapEngine.BackgroundFillBrush = New GeoSolidBrush(GeoColor.FromArgb(255, 233, 232, 214))

        Dim inMemoryFeatureLayer As New InMemoryFeatureLayer()
        inMemoryFeatureLayer.Open()
        inMemoryFeatureLayer.Columns.Add(New FeatureSourceColumn("Angle"))
        inMemoryFeatureLayer.Close()

        pointShapes1 = ReadTextFile("..\..\Data\GPSreadings1.txt")
        pointShapes2 = ReadTextFile("..\..\Data\GPSreadings2.txt")

        Dim newFeature1 As New Feature(pointShapes1(count1))
        newFeature1.ColumnValues("Angle") = System.Convert.ToString(pointShapes1(count1).Z)
        inMemoryFeatureLayer.InternalFeatures.Add("Vehicle1", newFeature1)

        Dim newFeature2 As New Feature(pointShapes2(count2))
        newFeature2.ColumnValues("Angle") = System.Convert.ToString(pointShapes2(count2).Z)
        inMemoryFeatureLayer.InternalFeatures.Add("Vehicle2", newFeature2)

        Dim rotatedImageStyle As New RotatedImageStyle(New GeoImage("..\..\Data\vehicle2.png"), "Angle")

        inMemoryFeatureLayer.ZoomLevelSet.ZoomLevel01.CustomStyles.Clear()
        inMemoryFeatureLayer.ZoomLevelSet.ZoomLevel01.CustomStyles.Add(rotatedImageStyle)
        inMemoryFeatureLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20

        mapEngine.DynamicLayers.Add(inMemoryFeatureLayer)
        mapEngine.CurrentExtent = New RectangleShape(-66.8581, 10.4995, -66.8456, 10.4915)

        timer1.Start()

        DrawImage()
    End Sub
End Class
